name = "Mediakit"
version = "0.6.0"
description = "Download YouTube videos fast, directly from the command line"
author = "Diego Aquino"
author_email = "diegocruzdeaquino@gmail.com"
url = "https://github.com/diego-aquino/mediakit"
download_url = "https://github.com/diego-aquino/mediakit/archive/v0.6.0.tar.gz"
license = "GPL-3.0"
copyright = "Copyright 2021 Diego Aquino"

temporary_filename = ".mediakit_download"
